<?php
/*
Plugin Name: pn-counter
Plugin URI: http://en.platonov.name/pn-counter/
Description: Visitor counter.
Author: Roman Platonov <r.v.platonov@gmail.com>
Contributor: Roman Platonov <r.v.platonov@gmail.com>
Author URI: http://en.platonov.name/
Version: 1.1
*/

function pn_counter_log($message) {
  $local_timeshift   = get_option('gmt_offset') * 60 * 60;
  $local_date        = time() + $local_timeshift;
  $local_date_string = gmdate("Y-m-d H:i:s", $local_date);
  
  $handle = fopen(ABSPATH."/pn_counter_log.txt", "a");
  fwrite($handle, $local_date_string." - ".$message."\n");
  fclose($handle);
}

function pn_counter_redirect($url) {
  echo("<meta http-equiv=\"refresh\" content=\"0;URL=$url\" />");
}

function pn_counter_install() {
  global $wpdb;
  
  $table_prefix = pn_counter_get_table_prefix();
  if ($table_prefix == null || $table_prefix == '') {
    return;
  }
  
  $stats_table_name = $table_prefix.'stats';
  if($wpdb->get_var("SHOW TABLES LIKE '$stats_table_name'") != $stats_table_name) {
    $sql = "CREATE TABLE  $stats_table_name (
      id bigint(20) unsigned NOT NULL auto_increment,
      local_date timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
      ip varchar(15) NOT NULL,
      url varchar(255) NOT NULL,
      referer varchar(255) default NULL,
      user_agent varchar(255) default NULL,
      hostname varchar(64) default NULL,
      uid bigint(20) unsigned default NULL,
      last_access bigint(20) unsigned default NULL,
      country_code varchar(2) default NULL,
      region varchar(32) default NULL,
      city varchar(32) default NULL,
      postal_code varchar(6) default NULL,
      latitude float default NULL,
      longitude float default NULL,
      area_code varchar(16) default NULL,
      dma_code varchar(16) default NULL,
      PRIMARY KEY (id)
    ) CHARACTER SET utf8 COLLATE utf8_general_ci;";
    $wpdb->query($sql);
  }
  
  $properties_table_name = $table_prefix.'properties';
  if($wpdb->get_var("SHOW TABLES LIKE '$properties_table_name'") != $properties_table_name) {
    $sql = "CREATE TABLE  $properties_table_name (
      id bigint(20) unsigned NOT NULL auto_increment,
      property_name varchar(255) NOT NULL,
      property_value varchar(255),
      PRIMARY KEY (id)
    ) CHARACTER SET utf8 COLLATE utf8_general_ci;";
    $wpdb->query($sql);
  }
  
  $last_migration = pn_counter_get_property('last_migration');
  
  // migration 1
  if ($last_migration == null || $last_migration == '') {
    $sql = "ALTER TABLE $stats_table_name ADD INDEX idx_".$stats_table_name."_local_date (local_date);";
    $wpdb->query($sql);
    pn_counter_put_property('last_migration', '1');
    $last_migration = 1;
  }
}

register_activation_hook(__FILE__, 'pn_counter_install');

//---------------------------------------------------------------------------------------------------------//

function pn_counter_get_table_prefix() {
  return get_option('pn-counter.table_prefix');
}

function pn_counter_set_table_prefix($table_prefix) {
  delete_option('pn-counter.table_prefix');
  add_option('pn-counter.table_prefix', $table_prefix);
}

function pn_counter_get_stats_table_name() {
  return get_option('pn-counter.table_prefix').'stats';
}

function pn_counter_get_property($property_name) {
  global $wpdb;
  
  $table_prefix = pn_counter_get_table_prefix();
  if ($table_prefix != null && $table_prefix != '') {
    $properties_table_name = $table_prefix.'properties';
    return $wpdb->get_var("SELECT property_value FROM $properties_table_name WHERE property_name = '$property_name';");
  } else {
    return null;
  }
}

function pn_counter_put_property($property_name, $property_value) {
  global $wpdb;
  
  $properties_table_name = get_option('pn-counter.table_prefix').'properties';
  $count = $wpdb->get_var("SELECT count(*) FROM $properties_table_name WHERE property_name = '$property_name';");
  if ($count == 1) {
    $wpdb->query("UPDATE $properties_table_name SET property_value = '$property_value' WHERE property_name = '$property_name';");
  } else {
    $wpdb->query("INSERT INTO $properties_table_name (property_name, property_value) VALUES ('$property_name', '$property_value');");
  }
}

function pn_counter_is_configured() {
  global $wpdb;
  
  $table_prefix = get_option('pn-counter.table_prefix');
  if ($table_prefix == null || $table_prefix == '') {
    return false;
  }
  
  $stats_table_name = $table_prefix.'stats';
  if($wpdb->get_var("SHOW TABLES LIKE '$stats_table_name'") != $stats_table_name) {
    return false;
  }
  
  $properties_table_name = $table_prefix.'properties';
  if($wpdb->get_var("SHOW TABLES LIKE '$properties_table_name'") != $properties_table_name) {
    return false;
  }
  
  return true;
}

function pn_counter_is_ip_excluded($ip) {
  global $wpdb;
  
  $excluded_ip_patterns = pn_counter_get_property('excluded_ip_patterns');
  if ($excluded_ip_patterns == null || $excluded_ip_patterns == '') {
    return false;
  }
  
  $excluded_ip_arr = split(" *; *", $excluded_ip_patterns);
  foreach ($excluded_ip_arr as $key => $value) {
    $pattern = str_replace("*", "\\d+", $value);
    $pattern = str_replace(".", "\\.", $pattern);
    $pattern = "/^".$pattern."$/";
    $matches = preg_match($pattern, $ip);
    if ($matches) {
      return true;
    }
  }
  
  return false;
}

function pn_counter_on_request() {
  if (!pn_counter_is_configured()) {
    return;
  }
  
  if ($_SERVER['HTTP_X_FORWARD_FOR']) {
    $ip = $_SERVER['HTTP_X_FORWARD_FOR'];
  } else {
    $ip = $_SERVER['REMOTE_ADDR'];
  }
  if (pn_counter_is_ip_excluded($ip)) {
    return;
  }
  
  $local_timeshift   = get_option("gmt_offset") * 60 * 60;
  $geocity_file_path = pn_counter_get_property('geocity_file_path');
  
  $local_date        = time() + $local_timeshift;
  $local_date_string = gmdate("Y-m-d H:i:s", $local_date);
  $url               = "http://".$_SERVER['SERVER_NAME'].$_SERVER['REQUEST_URI'];
  $referer           = $_SERVER['HTTP_REFERER'];
  $user_agent        = $_SERVER['HTTP_USER_AGENT'];
  $hostname          = null;
  $uid               = null;
  $last_access       = null;
  $country_code      = null;
  $region            = null;
  $city              = null;
  $postal_code       = null;
  $latitude          = null;
  $longitude         = null;
  $area_code         = null;
  $dma_code          = null;
  
  if (file_exists($geocity_file_path)) {
    require_once(ABSPATH.'wp-content/plugins/pn-counter/geoipcity.inc');
  
    $gi = geoip_open($geocity_file_path, GEOIP_STANDARD);
    $gir = GeoIP_record_by_addr($gi, $ip);
    
    if ($gir != null) {
      $country_code = $gir->country_code;
      $region       = $gir->region;
      $city         = $gir->city;
      $postal_code  = $gir->postal_code;
      $latitude     = $gir->latitude;
      $longitude    = $gir->longitude;
      $area_code    = $gir->area_code;
      $dma_code     = $gir->dma_code;
    }
  } else {
    pn_counter_log("ERROR: GeoCity file not found [$geocity_file_path]");
  }

  $stats_table_name = pn_counter_get_stats_table_name();
  $insert = "INSERT INTO $stats_table_name (local_date, ip, url";
  $values = "VALUES ('$local_date_string', '$ip', '$url'";
  if ($referer != null && $referer != '') {
    $insert .= ', referer';
    $values .= ', \''.mysql_real_escape_string($referer).'\'';
  }
  if ($user_agent != null && $user_agent != '') {
    $insert .= ', user_agent';
    $values .= ', \''.mysql_real_escape_string($user_agent).'\'';
  }
  if ($hostname != null && $hostname != '') {
    $insert .= ', hostname';
    $values .= ', \''.mysql_real_escape_string($hostname).'\'';
  }
  if ($uid != null && $uid != '') {
    $insert .= ', uid';
    $values .= ', '.mysql_real_escape_string($uid);
  }
  if ($last_access != null && $last_access != '') {
    $insert .= ', last_access';
    $values .= ', '.mysql_real_escape_string($last_access);
  }
  if ($country_code != null && $country_code != '') {
    $insert .= ', country_code';
    $values .= ', \''.mysql_real_escape_string($country_code).'\'';
  }
  if ($region != null && $region != '') {
    $insert .= ', region';
    $values .= ', \''.mysql_real_escape_string($region).'\'';
  }
  if ($city != null && $city != '') {
    $insert .= ', city';
    $values .= ', \''.mysql_real_escape_string($city).'\'';
  }
  if ($postal_code != null && $postal_code != '') {
    $insert .= ', postal_code';
    $values .= ', \''.mysql_real_escape_string($postal_code).'\'';
  }
  if ($latitude != null && $latitude != '') {
    $insert .= ', latitude';
    $values .= ', '.mysql_real_escape_string($latitude);
  }
  if ($longitude != null && $longitude != '') {
    $insert .= ', longitude';
    $values .= ', '.mysql_real_escape_string($longitude);
  }
  if ($area_code != null && $area_code != '') {
    $insert .= ', area_code';
    $values .= ', \''.mysql_real_escape_string($area_code).'\'';
  }
  if ($dma_code != null && $dma_code != '') {
    $insert .= ', dma_code';
    $values .= ', \''.mysql_real_escape_string($dma_code).'\'';
  }
  $insert .= ') ';
  $values .= ');';
  
  global $wpdb;
  $wpdb->query($insert.$values);
}

add_action('init', 'pn_counter_on_request');


function pn_counter_add_menu() {
  $configured = pn_counter_is_configured();
  if ($configured) {
    $default_file = ABSPATH.'wp-content/plugins/pn-counter/raw-values.php';
  } else {
    $default_file = ABSPATH.'wp-content/plugins/pn-counter/settings.php';
  }
  add_menu_page('pn-counter', 'pn-counter', 8, $default_file);
  if ($configured) {
  	add_submenu_page($default_file, 'pn-counter: raw values', 'raw values', 8, ABSPATH.'wp-content/plugins/pn-counter/raw-values.php');
  }
	add_submenu_page($default_file, 'pn-counter: settings', 'settings', 8, ABSPATH.'wp-content/plugins/pn-counter/settings.php');
}

add_action('admin_init', 'pn_counter_add_menu');

?>